<?php 
/**
 * @file
 * Alternative page for the search module, for the user search feature, 
 * searching by email addresses.
 */

/**
 * Menu callback; presents the search form and/or search results.
 *
 * Adapted from the search_view() function, in 
 * 'module/search/search.pages.inc', line 11
 */
function dbee_search_view($type = 'node') {
  // Search form submits with POST but redirects to GET. This way we can keep
  // the search query URL clean as a whistle:
  // search/type/keyword+keyword
  if (!isset($_POST['form_id'])) {
    if ($type == '') {
      // Note: search/node can not be a default tab because it would take on the
      // path of its parent (search). It would prevent remembering keywords when
      // switching tabs. This is why we drupal_goto to it from the parent instead.
      drupal_goto('search/node');
    }
    
    // Function search_get_keys() is available in the "search.module" file, 
    // no need to include it.
    $keys = search_get_keys();
    // Only perform search if there is non-whitespace search term:
    $results = '';
    if (trim($keys)) {
      // Log the search keys:
      watchdog('search', '%keys (@type).', array('%keys' => $keys, '@type' => dbee_user_search('name')), WATCHDOG_NOTICE, l(t('results'), 'search/' . $type . '/' . $keys));

      // Collect the search results:
      // HERE we hijack the search_data function
      $results = dbee_search_data($keys, $type);
      
      if ($results) {
        $results = theme('box', t('Search results'), $results);
      }
      else {
        // Function search_help() is available in the "search.module" file, no 
        // need to include it.
        $results = theme('box', t('Your search yielded no results'), search_help('search#noresults', drupal_help_arg()));
      }
    }

    // Construct the search form.
    $output = drupal_get_form('search_form', NULL, $keys, $type);
    $output .= $results;

    return $output;
  }

  return drupal_get_form('search_form', NULL, empty($keys) ? '' : $keys, $type);
}


/**
 * Perform a standard search on the given keys, and return the formatted results.
 *
 * Adapted from the search_data() function, in 'module/search/search.module', 
 * line 1153
 */
function dbee_search_data($keys = NULL, $type = 'node') {

  if (isset($keys)) {
    // HERE is the primary goal : We don't want to call user_search($op = 
    //'search') : we call an alternative function for correct search 
    $results = dbee_user_search('search', $keys);
    if (isset($results) && is_array($results) && count($results)) {
      if (module_hook($type, 'search_page')) {
        return module_invoke($type, 'search_page', $results);
      }
      else {
        return theme('search_results', $results, $type);
      }
    }
  }
}


/**
 * Procced to the user search.
 *
 * Adapted from the user_search() function, in 'module/user/user.module', 
 * line 573.
 * The dbee specific feature is enable in a unique specific situation : 
 * - search module is enable,
 * - and ask for user search page (url ?q=search/user) 
 * - and logged user has "administer users" permission.
 */
function dbee_user_search($op = 'search', $keys = NULL, $skip_access_check = FALSE) {
  switch ($op) {
    case 'name':
      if ($skip_access_check || user_access('access user profiles')) {
        return t('Users');
      }
    case 'search':
      if (user_access('access user profiles')) {
        $find = array();
        // Replace wildcards with MySQL/PostgreSQL wildcards.
        $keys = preg_replace('!\*+!', '%', $keys);
        if (user_access('administer users')) {
          // Administrators can also search in the otherwise private email field.
          // HERE : encrypt email address for searching
          $result = pager_query("SELECT name, uid, mail FROM {users} WHERE LOWER(name) LIKE LOWER('%%%s%%') OR mail LIKE '%%%s%%'", 15, 0, NULL, $keys, dbee_convert($keys));
          while ($account = db_fetch_object($result)) {
          // HERE : decrypt email address for sensitive case stored value.
            $find[] = array('title' => $account->name . ' (' . dbee_case_load($account->mail, $account->uid) . ')', 'link' => url('user/' . $account->uid, array('absolute' => TRUE)));
          }
        }
        else {
          $result = pager_query("SELECT name, uid FROM {users} WHERE LOWER(name) LIKE LOWER('%%%s%%')", 15, 0, NULL, $keys);
          while ($account = db_fetch_object($result)) {
            $find[] = array('title' => $account->name, 'link' => url('user/' . $account->uid, array('absolute' => TRUE)));
          }
        }
        return $find;
      }
  }
}